import app.pe.pe_comp as pe_comp
import app.pe.pe_affichage as pe_affichage

from app.models import FormSemestre
from app.pe.pe_etudiant import EtudiantsJuryPE, get_dernier_semestre_en_date


class Trajectoire:
    def __init__(self, nom_aggregat: str, semestre_final: FormSemestre):
        """Modélise un ensemble de formsemestres d'étudiants
        amenant à un semestre terminal, au sens d'un aggrégat (par ex: 'S2', '3S', '2A').

        Si l'aggrégat est un semestre de type Si, elle stocke le (ou les)
        formsemestres de numéro i qu'ont suivi l'étudiant pour atteindre le Si
        (en général 1 si personnes n'a redoublé, mais 2 s'il y a des redoublants)

        Pour des aggrégats de type iS ou iA (par ex, 3A=S1+S2+S3), elle identifie
        les semestres que les étudiants ont suivis pour les amener jusqu'au semestre
        terminal de la trajectoire (par ex: ici un S3).

        Ces semestres peuvent être :

        * des S1+S2+S1+S2+S3 si redoublement de la 1ère année
        * des S1+S2+(année de césure)+S3 si césure, ...

        Args:
            nom_aggregat: Un nom d'aggrégat (par ex: '5S')
            semestre_final: Le semestre final de l'aggrégat
        """
        self.nom = nom_aggregat
        """Nom de l'aggrégat"""

        self.formsemestre_final = semestre_final
        """FormSemestre terminal de la trajectoire"""

        self.trajectoire_id = (nom_aggregat, semestre_final.formsemestre_id)
        """Identifiant de la trajectoire"""

        self.semestres_aggreges = {}
        """Semestres aggrégés"""

    def add_semestres_a_aggreger(self, semestres: dict[int:FormSemestre]):
        """Ajout de semestres aux semestres à aggréger

        Args:
            semestres: Dictionnaire ``{fid: FormSemestre(fid)}`` à ajouter
        """
        self.semestres_aggreges = self.semestres_aggreges | semestres

    def get_repr(self, verbose=True) -> str:
        """Représentation textuelle d'une trajectoire
        basée sur ses semestres aggrégés"""

        noms = []
        for fid in self.semestres_aggreges:
            semestre = self.semestres_aggreges[fid]
            noms.append(f"S{semestre.semestre_id}({fid})")
        noms = sorted(noms)
        repr = f"{self.nom} ({self.formsemestre_final.formsemestre_id}) {self.formsemestre_final.date_fin.year}"
        if verbose and noms:
            repr += " - " + "+".join(noms)
        return repr


class TrajectoiresJuryPE:
    def __init__(self, annee_diplome: int):
        """Classe centralisant toutes les trajectoires des étudiants à prendre
        en compte dans un jury PE

        Args:
            annee_diplome: L'année de diplomation
        """

        self.annee_diplome = annee_diplome
        """Toutes les trajectoires possibles"""

        self.trajectoires: dict[tuple:Trajectoire] = {}
        """Ensemble des trajectoires recensées : {(aggregat, fid_terminal): Trajectoire}"""

        self.suivi: dict[int:str] = {}
        """Dictionnaire associant, pour chaque étudiant et pour chaque aggrégat,
            sa trajectoire : {etudid: {nom_aggregat: Trajectoire}}"""

    def cree_trajectoires(self, etudiants: EtudiantsJuryPE):
        """Créé toutes les trajectoires, au regard du cursus des étudiants
        analysés + les mémorise dans les données de l'étudiant
        """

        for nom_aggregat in pe_comp.TOUS_LES_SEMESTRES + pe_comp.TOUS_LES_AGGREGATS:
            """L'aggrégat considéré (par ex: 3S=S1+S2+S3), son nom de son semestre terminal (par ex: S3) et son numéro (par ex: 3)"""
            noms_semestre_de_aggregat = pe_comp.PARCOURS[nom_aggregat]["aggregat"]
            nom_semestre_terminal = noms_semestre_de_aggregat[-1]

            for etudid in etudiants.cursus:
                if etudid not in self.suivi:
                    self.suivi[etudid] = {
                        aggregat: None
                        for aggregat in pe_comp.TOUS_LES_SEMESTRES
                        + pe_comp.TOUS_LES_AGGREGATS
                    }

                """Le formsemestre terminal (dernier en date) associé au 
                semestre marquant la fin de l'aggrégat 
                (par ex: son dernier S3 en date)"""
                semestres = etudiants.cursus[etudid][nom_semestre_terminal]
                if semestres:
                    formsemestre_final = get_dernier_semestre_en_date(semestres)

                    """Ajout ou récupération de la trajectoire"""
                    trajectoire_id = (nom_aggregat, formsemestre_final.formsemestre_id)
                    if trajectoire_id not in self.trajectoires:
                        trajectoire = Trajectoire(nom_aggregat, formsemestre_final)
                        self.trajectoires[trajectoire_id] = trajectoire
                    else:
                        trajectoire = self.trajectoires[trajectoire_id]

                    """La liste des semestres de l'étudiant à prendre en compte
                    pour cette trajectoire"""
                    semestres_a_aggreger = etudiants.get_trajectoire(
                        etudid, formsemestre_final, nom_aggregat
                    )

                    """Ajout des semestres à la trajectoire"""
                    trajectoire.add_semestres_a_aggreger(semestres_a_aggreger)

                    """Mémoire la trajectoire suivie par l'étudiant"""
                    self.suivi[etudid][nom_aggregat] = trajectoire



def get_trajectoires_etudid(trajectoires, etudid):
    """Fonction pour débuggage: renvoie la liste des trajectoires_id des
    trajectoires suivies par un étudiant
    """
    if etudid not in trajectoires.suivi:
        pe_affichage.pe_print(f"{etudid} fait-il bien partie du jury ?")

    liste = []
    for aggregat in pe_comp.TOUS_LES_PARCOURS:
        trajet = trajectoires.suivi[etudid][aggregat]
        if trajet:
            liste.append(trajet.trajectoire_id)
    return liste


